<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Action;

use Rebilly\Wp\Member\Entity\User;
use Rebilly\Wp\Member\Plugin;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class InvoiceDownload extends AjaxAction
{
    public function __construct()
    {
        parent::__construct('rebilly_invoice_download', false);
    }

    /**
     * @param array $data
     *
     * @return array
     */
    public function process(array $data): array
    {
        $user = User::find();
        $invoiceId = (string) $data['invoiceId'];

        $invoice = Plugin::api()->invoice()->get($invoiceId);

        if (!$invoice || $invoice->getCustomerId() !== $user->getCustomerId()) {
            wp_redirect(home_url());
            wp_die();
        }

        $pdf = Plugin::api()->invoice()->getPdf($invoiceId);

        if (!$pdf) {
            wp_redirect(home_url());
            wp_die();
        }

        header($_SERVER['SERVER_PROTOCOL'] . ' 200 OK');
        header('Cache-Control: private');
        header("Content-Type: {$pdf->getMimeType()}");
        header('Content-Transfer-Encoding: Binary');
        header("Content-Length:{$pdf->getSize()}");
        header("Content-Disposition: attachment; filename={$invoiceId}.pdf");
        echo $pdf->getContent();
        wp_die();

        return [];
    }
}
