<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Action;

use Rebilly\Wp\Member\Entity\Subscription;
use Rebilly\Wp\Member\Entity\User;
use Rebilly\Wp\Member\Plugin;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class InvoicesPaginate extends AjaxAction
{
    public function __construct()
    {
        parent::__construct('rebilly_invoices_paginate', false);
    }

    /**
     * @param array $data
     * @return array
     */
    public function process(array $data): array
    {
        $user = User::find();

        $perPage = (int) apply_filters('rebilly_invoices_per_page', 10);
        $page = max((int) ($data['page'] ?? 1), 1);

        $invoices = [];

        if ($user->getCustomerId()) {
            $subscriptionId = '';
            if (!blank($data['subscriptionId'] ?? '') && $subscription = Subscription::find($data['subscriptionId'])) {
                $subscriptionId = $subscription->getSubscriptionId();
            }

            $invoicesRaw = Plugin::api()
                ->invoice()
                ->findByCustomer($user->getCustomerId(), $subscriptionId, $page, $perPage);

            $subscriptionIds = [];
            foreach ($invoicesRaw as $invoice) {
                $subscriptionIds[] = $invoice->getSubscriptionId();
            }
            $subscriptionIds = array_unique(array_filter($subscriptionIds));
            $subscriptionsRaw = Subscription::findAll(['subscriptionId' => $subscriptionIds], true);
            $subscriptions = [];
            foreach ($subscriptionsRaw as $subscription) {
                $subscriptions[$subscription->getSubscriptionId()] = $subscription;
            }
            foreach ($invoicesRaw as $invoice) {
                $items = [];
                $itemsRaw = Plugin::api()
                    ->invoice()
                    ->findItems($invoice->getId());

                foreach ($itemsRaw as $item) {
                    $items[] = [
                        'type' => $item->getType(),
                        'name' => $item->getDescription(),
                        'unitPrice' => $item->getUnitPrice() ?? 1,
                        'quantity' => $item->getQuantity(),
                        'price' => ($item->getUnitPrice() ?? 1) * $item->getQuantity(),
                    ];
                }
                if ($invoice->getShippingAmount() > 0) {
                    $items[] = [
                        'type' => 'debit',
                        'name' => __('Shipping', REBILLY_WP_LANG_DOMAIN),
                        'unitPrice' => $invoice->getShippingAmount(),
                        'quantity' => 1,
                        'price' => $invoice->getShippingAmount(),
                    ];
                }
                $invoices[$invoice->getId()] = [
                    'id' => $invoice->getId(),
                    'name' => $invoice->getInvoiceNumber(),
                    'currency' => $invoice->getCurrency(),
                    'amount' => $invoice->getAmount(),
                    'status' => $invoice->getStatus(),
                    'issuedTime' => strtotime($invoice->getIssuedTime() ?? ''),
                    'paidTime' => strtotime($invoice->getPaidTime() ?? ''),
                    'dueTime' => strtotime($invoice->getDueTime() ?? ''),
                    'items' => $items,
                ];
            }
            $pageCount = ceil($invoicesRaw->getTotalItems() / $perPage);
        }

        return [
            'success' => true,
            'items' => renderIsolate(REBILLY_WP_DIR . 'template/front/ajax/invoice-list.php', compact('invoices'), true),
            'page' => $page,
            'pageCount' => $pageCount ?? 0,
        ];
    }
}
