<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

namespace Rebilly\Wp\Member\Admin;

use Rebilly\Wp\Member\Admin\Block\Block;
use Rebilly\Wp\Member\Admin\Block\DynamicPageShortcode;
use Rebilly\Wp\Member\Admin\Block\FormShortcode;
use Rebilly\Wp\Member\Admin\Block\PortalShortcode;
use Rebilly\Wp\Member\Admin\Page\SettingsPage;
use Rebilly\Wp\Member\Entity\Form as FormEntity;
use Rebilly\Wp\Member\Plugin;
use Rebilly\Wp\Member\Shortcode\DynamicPage;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Admin
{
    /** @var Block[]|array $blocks */
    private static $blocks = [
        FormShortcode::class => false,
        DynamicPageShortcode::class => false,
        PortalShortcode::class => false,
    ];

    private $form;

    private $restrictable;

    private $user;

    /**
     * WP_Rebilly_Member_Admin constructor.
     */
    public function __construct()
    {
        if (Plugin::instance()->isConnected()) {
            $this->form = new Form();
            $this->restrictable = new RestrictableType();
            $this->user = new User();
        }

        if (!Plugin::instance()->isConnected()) {
            rebilly_admin_notice('Please, check your Rebilly API keys at settings page to enable Rebilly.', 'error');
        }

        SettingsPage::register();
        $this->loadHooks();
        $this->loadBlocks();
    }

    /**
     * @return Form
     */
    public function getForm(): Form
    {
        return $this->form;
    }

    /**
     * @return RestrictableType
     */
    public function getRestrictable(): RestrictableType
    {
        return $this->restrictable;
    }

    /**
     * @return User
     */
    public function getUser(): User
    {
        return $this->user;
    }

    public function loadAssets(): void
    {
        wp_enqueue_script('wp-color-picker');
        wp_enqueue_style('wp-color-picker');

        wp_register_style('rebilly-admin', REBILLY_WP_URL . 'dist/css/admin.css', ['wp-color-picker'], filemtime(REBILLY_WP_DIR . 'dist/css/admin.css'));
        wp_register_script('rebilly-admin', REBILLY_WP_URL . 'dist/js/admin.js', ['wp-color-picker'], filemtime(REBILLY_WP_DIR . 'dist/js/admin.js'), true);

        wp_localize_script('rebilly-admin', 'rebilly_admin', [
            'canvasStyles' => [REBILLY_WP_URL . 'dist/css/canvas.css'],
            'pages' => DynamicPage::getPageNames(),
            'customFields' => FormEntity::getCustomFieldsOptions(),
        ]);

        wp_enqueue_style('rebilly-admin');
        wp_enqueue_script('rebilly-admin');
    }

    public function clearUsersCache(): void
    {
        wp_rebilly_m_cached('user-names', false);
    }

    private function loadHooks(): void
    {
        add_action('admin_enqueue_scripts', [$this, 'loadAssets']);
        add_action('user_register', [$this, 'clearUsersCache']);
    }

    private function loadBlocks(): void
    {
        if (!function_exists('register_block_type')) {
            // Gutenberg isn't available.
            return;
        }
        foreach (static::$blocks as $block => &$instance) {
            if ($instance !== false) {
                continue;
            }

            $obj = new $block();
            if ($obj instanceof Block) {
                $obj->register();
            }
        }
    }
}
