<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Admin\Block;

use Rebilly\Wp\Member\Entity\Form;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class FormShortcode extends Block
{
    public function __construct()
    {
        $this->name = 'form-shortcode';
    }

    public function registerOptions(): array
    {
        $options = parent::registerOptions();
        $options['render_callback'] = [$this, 'renderCallback'];

        return $options;
    }

    /**
     * @param array $attributes
     * @return string
     */
    public function renderCallback(array $attributes): string
    {
        $shortcode = ['wp_rebilly_m_form'];
        $form = isset($attributes['formId']) ? Form::find($attributes['formId']) : false;
        if (!$form) {
            return __('You must select form first', REBILLY_WP_LANG_DOMAIN);
        }
        $shortcode[] = $form->ID;
        if (in_array($attributes['condition'] ?? '', ['only:auth', 'only:guest'], true)) {
            $shortcode[] = $attributes['condition'];
        }
        $shortcode = implode(' ', $shortcode);

        return do_shortcode("[{$shortcode}]");
    }
}
