<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Api\Middleware;

use Psr\Http\Message\RequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use Rebilly\Middleware;
use Rebilly\Wp\Member\Api\Client;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class HandleUnauthorized implements Middleware
{
    /** @var Client */
    private $client;

    /**
     * HandleUnauthorized constructor.
     * @param Client $client
     */
    public function __construct(Client $client)
    {
        $this->client = $client;
    }

    /**
     * @inheritdoc
     */
    public function __invoke(Request $request, Response $response, callable $next)
    {
        $result = call_user_func($next, $request, $response);
        if ($result instanceof Response) {
            $response = $result;
        }

        if ($response->getStatusCode() === 401) {
            $this->client->setUnauthorized(true);
        }

        return $response;
    }
}
