<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Integration\Elementor;

use Elementor\Controls_Manager;
use Elementor\Widget_Base;
use Rebilly\Wp\Member\Shortcode\DynamicPage;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class DynamicPageWidget extends Widget_Base
{
    protected $pages;

    public function __construct(array $data = [], ?array $args = null)
    {
        parent::__construct($data, $args);

        $this->pages = DynamicPage::getPageNames(true);
    }

    /**
     * Get element name.
     *
     * Retrieve the element name.
     *
     * @since 1.4.0
     * @access public
     *
     * @return string The name.
     */
    public function get_name(): string
    {
        return 'rebilly-dynamic-page';
    }

    public function get_title(): string
    {
        return __('Rebilly Page', REBILLY_WP_LANG_DOMAIN);
    }

    public function get_icon(): string
    {
        return 'fa fa-file';
    }

    public function get_keywords(): array
    {
        return ['rebilly', 'page', 'subscription', 'invoice'];
    }

    protected function _register_controls(): void
    {
        $this->start_controls_section(
            'settings_section',
            [
                'label' => __('Settings', REBILLY_WP_LANG_DOMAIN),
                'tab' => Controls_Manager::TAB_SETTINGS,
            ]
        );

        $this->add_control(
            'pageId',
            [
                'label' => __('Page Type', REBILLY_WP_LANG_DOMAIN),
                'type' => Controls_Manager::SELECT2,
                'label_block' => true,
                'options' => $this->pages,
            ]
        );

        $this->end_controls_section();
    }

    protected function render(): void
    {
        $settings = $this->get_settings_for_display();
        $shortcode = array_filter([
            DynamicPage::NAME,
            $settings['pageId'] ?? null,
        ]);

        echo do_shortcode('[' . implode(' ', $shortcode) . ']');
    }

    protected function _content_template(): void
    {
        renderIsolate(REBILLY_WP_DIR . 'template/admin/integration/elementor/dynamic-page.php', [
            'pages' => $this->pages,
        ]);
    }
}
