<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Integration\Elementor;

use Rebilly\Wp\Member\Entity\Form;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class FormWidget extends \Elementor\Widget_Base
{
    protected $forms = [];

    protected $conditions;

    public function __construct(array $data = [], ?array $args = null)
    {
        parent::__construct($data, $args);

        $this->conditions = [
            '' => __('Display for all', REBILLY_WP_LANG_DOMAIN),
            'only:auth' => __('Members only', REBILLY_WP_LANG_DOMAIN),
            'only:guest' => __('Guests only', REBILLY_WP_LANG_DOMAIN),
        ];
    }

    /**
     * @inheritdoc
     */
    public function get_name(): string
    {
        return 'rebilly-form';
    }

    public function get_title(): string
    {
        return __('Rebilly Form', REBILLY_WP_LANG_DOMAIN);
    }

    public function get_icon(): string
    {
        return 'fa fa-list';
    }

    public function get_categories(): array
    {
        return ['general'];
    }

    public function get_keywords()
    {
        return ['rebilly', 'form', 'subscription'];
    }

    protected function _register_controls()
    {
        $this->start_controls_section(
            'settings_section',
            [
                'label' => __('Settings', REBILLY_WP_LANG_DOMAIN),
                'tab' => \Elementor\Controls_Manager::TAB_SETTINGS,
            ]
        );

        $this->add_control(
            'formId',
            [
                'label' => __('Form', REBILLY_WP_LANG_DOMAIN),
                'type' => \Elementor\Controls_Manager::SELECT2,
                'label_block' => true,
                'options' => $this->getForms(),
            ]
        );

        $this->add_control(
            'condition',
            [
                'label' => __('Condition', REBILLY_WP_LANG_DOMAIN),
                'type' => \Elementor\Controls_Manager::SELECT2,
                'label_block' => true,
                'options' => $this->conditions,
            ]
        );

        $this->end_controls_section();
    }

    protected function getForms()
    {
        if (count($this->forms) === 0) {
            $formsRaw = Form::all() ?? [];
            $this->forms = [
                '' => __('Select a Form', REBILLY_WP_LANG_DOMAIN),
            ];
            foreach ($formsRaw as $form) {
                $this->forms[$form->ID] = get_the_title($form);
            }
        }

        return $this->forms;
    }

    protected function render()
    {
        // TODO force preview mode for editing
        $settings = $this->get_settings_for_display();
        $shortcode = array_filter([
            'wp_rebilly_m_form',
            $settings['formId'] ?? null,
            $settings['condition'] ?? null,
        ]);

        echo do_shortcode('[' . implode(' ', $shortcode) . ']');
    }

    protected function _content_template()
    {
        renderIsolate(REBILLY_WP_DIR . 'template/admin/integration/elementor/form.php', [
            'forms' => $this->getForms(),
            'conditions' => $this->conditions,
        ]);
    }
}
