<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Integration;

// Exit if accessed directly.
use Rebilly\Wp\Member\Integration\SiteOrigin\DynamicPageWidget;
use Rebilly\Wp\Member\Integration\SiteOrigin\FormWidget;
use Rebilly\Wp\Member\Integration\SiteOrigin\PortalWidget;

if (!defined('ABSPATH')) {
    exit();
}

class SiteOrigin implements IntegrationContract
{
    public static $widgets = [
        FormWidget::class,
        DynamicPageWidget::class,
        PortalWidget::class,
    ];

    public static function isApplicable(): bool
    {
        return class_exists('SiteOrigin_Widget');
    }

    public function register(): void
    {
        add_action('widgets_init', [$this, 'init']);
        add_filter('siteorigin_panels_widget_dialog_tabs', [$this, 'addTabs'], 20);
    }

    public function init(): void
    {
        global $wp_widget_factory;
        foreach (static::$widgets as $widget) {
            register_widget($widget);

            // Workaround for SiteOrigin preview
            if (($_REQUEST['action'] ?? '') === 'so_widgets_preview' && wp_doing_ajax()) {
                $wp_widget_factory->widgets[str_replace('\\', '\\\\', $widget)] = &$wp_widget_factory->widgets[$widget];
            }
        }
        wp_unregister_sidebar_widget('rebilly_form');
        wp_unregister_sidebar_widget('rebilly_dynamic_page');
        wp_unregister_sidebar_widget('rebilly_portal');
    }

    public function addTabs($tabs)
    {
        $tabs[] = [
            'title' => __('Rebilly', REBILLY_WP_LANG_DOMAIN),
            'filter' => [
                'groups' => ['rebilly'],
            ],
        ];

        return $tabs;
    }
}
