<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Integration\SiteOrigin;

use Rebilly\Wp\Member\Shortcode\DynamicPage;
use SiteOrigin_Widget;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class DynamicPageWidget extends SiteOrigin_Widget
{
    public function __construct()
    {
        parent::__construct(
            'rebilly_dynamic_page',
            __('Rebilly Page', REBILLY_WP_LANG_DOMAIN),
            [
                'description' => __('Insert a Rebilly dynamic page on your page', REBILLY_WP_LANG_DOMAIN),
                'panels_icon' => 'dashicons dashicons-admin-page',
                'panels_groups' => ['rebilly'],
            ],
            [],
            [
                'pageId' => [
                    'type' => 'select',
                    'label' => __('Page', REBILLY_WP_LANG_DOMAIN),
                    'default' => '',
                    'prompt' => __('Select a Page Type', REBILLY_WP_LANG_DOMAIN),
                    'input_css_classes' => ['widefat'],
                    'options' => DynamicPage::getPageNames(true),
                ],
            ]
        );
        add_action('siteorigin_widgets_render_preview_' . $this->id_base, [$this, 'enqueueStyles']);
        add_filter('siteorigin_widgets_template_file_' . $this->id_base, [$this, 'getTemplateFile']);
    }

    public function enqueueStyles(): void
    {
        wp_enqueue_style('wp_rebilly_m-front', REBILLY_WP_URL . 'dist/css/front.css', [], '1.0.0');
    }

    public function getTemplateFile(): string
    {
        return REBILLY_WP_DIR . 'template/admin/integration/siteorigin/dynamic-page.php';
    }
}
