<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Integration\Thrive;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Form extends \TCB_Element_Abstract
{
    public function __construct(string $tag = '')
    {
        parent::__construct($tag);

        add_filter('tcb_modal_templates', [$this, 'addModal']);
    }

    public function addModal(array $modals): array
    {
        $modals[] = REBILLY_WP_DIR . 'template/admin/integration/thrive/form-model.php';

        return $modals;
    }

    /**
     * Get element alternate
     *
     * @return string
     */
    public function alternate()
    {
        return 'rebilly-form';
    }

    /**
     * Name of the element
     *
     * @return string
     */
    public function name()
    {
        return __('Rebilly Form', REBILLY_WP_LANG_DOMAIN);
    }

    /**
     * Return icon class needed for display in menu
     *
     * @return string
     */
    public function icon()
    {
        return 'feedback';
    }

    /**
     * Button element identifier
     *
     * @return string
     */
    public function identifier()
    {
        return '.thrv-rebilly_form, .thrv_rebilly_form_shortcode';
    }

    public function html()
    {
        return $this->html_placeholder('Select Rebilly Form');
    }

    /**
     * Component and control config
     *
     * @return array
     */
    public function own_components()
    {
        return [
            'rebilly' => [
                'config' => [
                    'form' => 'test',
                ],
            ],
            'typography' => ['hidden' => true],
            'borders' => ['hidden' => true],
            'animation' => ['hidden' => true],
            'background' => ['hidden' => true],
            'shadow' => ['hidden' => true],
        ];
    }

    public function is_placeholder()
    {
        return true; // TODO: Change the autogenerated stub
    }

    /**
     * Element category that will be displayed in the sidebar
     *
     * @return string
     */
    public function category()
    {
        return $this->get_thrive_integrations_label();
    }
}
