<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Reformer;

use Rebilly\Wp\Member\Reformer\Traits\HasAttributes;
use Rebilly\Wp\Member\Reformer\Traits\HasChildren;
use Rebilly\Wp\Member\Reformer\Traits\HasShowFor;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Divider implements Node
{
    use HasAttributes, HasChildren, HasShowFor;

    protected $tag = '';

    protected $config = [];

    /**
     * Label constructor.
     * @param array $config
     */
    public function __construct(array $config)
    {
        $this->config = $config;

        $this->tag = 'hr';

        $this->children = Reformer::buildNodes($config['children'] ?? []);

        $this->attributes['id'] = $config['id'];
        $this->attributes['class'] = $config['class'];

        $condition = $config['settings']['condition'] ?? '';
        if (!blank($condition)) {
            $this->attributes['data-condition'] = !is_string($condition)? json_encode($condition) : $condition;
        }
    }

    public function getTag(): string
    {
        return $this->tag;
    }

    public function render(array $env): string
    {
        if (!$this->isAvailable($env)) {
            return '';
        }

        [$tag, $attributesRaw] = [$this->getTag(), $this->getAttributes()];

        // Filter empty ones
        $attributesRaw = array_filter($attributesRaw, function ($value) {
            return !blank($value);
        });

        // Compile attributes
        $attributes = implode(' ', array_map(function ($keyRaw, $valueRaw) {
            [$key, $value] = [esc_attr($keyRaw), esc_attr($valueRaw)];

            return "$key=\"$value\"";
        }, array_keys($attributesRaw), $attributesRaw));

        return "<$tag $attributes/>";
    }
}
