<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Reformer;

use Rebilly\Wp\Member\Reformer\Traits\HasAttributes;
use Rebilly\Wp\Member\Reformer\Traits\HasChildren;
use Rebilly\Wp\Member\Reformer\Traits\HasDomRenderer;
use Rebilly\Wp\Member\Reformer\Traits\HasShowFor;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Option implements Node
{
    use HasAttributes, HasChildren, HasDomRenderer, HasShowFor;

    protected $config = [];

    /**
     * Label constructor.
     * @param array $config
     */
    public function __construct(array $config)
    {
        $this->config = $config;

        $this->children = Reformer::buildNodes($config['children'] ?? []);

        foreach ($config['attributes'] as $key => $value) {
            if (blank($value)) {
                continue;
            }
            if (in_array($key, ['value'], true)) {
                $this->attributes[$key] = $value;
            } elseif (in_array($key, ['selected'], true)) {
                $this->attributes[$key] = $key;
            }
        }
    }

    public function getTag(): string
    {
        return 'option';
    }

    public function getKey(): string
    {
        return $this->attributes['value'] ?? '';
    }

    public function getLabel(): string
    {
        return $this->renderInner([]);
    }

    /**
     * @param array $env
     *
     * @return string
     */
    public function render(array $env): string
    {
        if (!$this->isAvailable($env)) {
            return '';
        }

        [$tag, $attributesRaw, $inner] = [$this->getTag(), $this->getAttributes(), $this->renderInner($env)];

        // Filter empty ones
        $attributesRaw = array_filter($attributesRaw, function ($value) {
            return !blank($value);
        });
        if (!isset($attributesRaw['value'])) {
            $attributesRaw['value'] = '';
        }
        if (!empty($env['rebilly-role']) &&
            isset($env['preFill'][$env['rebilly-role']]) &&
            $env['preFill'][$env['rebilly-role']] === $attributesRaw['value']) {
            $attributesRaw['selected'] = 'selected';
        }

        // Compile attributes
        $attributes = implode(' ', array_map(function ($keyRaw, $valueRaw) {
            [$key, $value] = [esc_attr($keyRaw), esc_attr($valueRaw)];

            return "$key=\"$value\"";
        }, array_keys($attributesRaw), $attributesRaw));

        return "<$tag $attributes>$inner</$tag>";
    }
}
