<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Reformer;

use Rebilly\Wp\Member\Reformer\Traits\HasAttributes;
use Rebilly\Wp\Member\Reformer\Traits\HasChildren;
use Rebilly\Wp\Member\Reformer\Traits\HasDomRenderer;
use Rebilly\Wp\Member\Reformer\Traits\HasShowFor;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Textarea implements Node
{
    use HasAttributes, HasChildren, HasDomRenderer, HasShowFor;

    protected $config = [];

    /**
     * Node constructor.
     *
     * @param array $config
     */
    public function __construct(array $config)
    {
        $this->config = $config;

        $this->children = Reformer::buildNodes($config['children'] ?? []);

        $this->attributes['id'] = $config['id'];
        $this->attributes['class'] = $config['class'];
        foreach ($config['attributes'] as $key => $value) {
            if (blank($value)) {
                continue;
            }
            if (in_array($key, ['name', 'value', 'type', 'placeholder'], true)) {
                $this->attributes[$key] = $value;
            } elseif (in_array($key, ['required', 'checked'], true)) {
                $this->attributes[$key] = $key;
            }
        }

        if (!blank($config['settings']['rebilly'] ?? '')) {
            $this->attributes['data-rebilly'] = $config['settings']['rebilly'];
        }
    }

    /**
     * @return string
     */
    public function getTag(): string
    {
        return 'textarea';
    }
}
