<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Shortcode;

use Rebilly\Entities\Customer;
use Rebilly\Entities\PaymentInstruments\PaymentCardInstrument;
use Rebilly\Wp\Member\Action\Action;
use Rebilly\Wp\Member\Entity\User;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Portal implements ShortcodeContract
{
    /**
     * @return string
     */
    public static function getName(): string
    {
        return 'rebilly_portal';
    }

    /**
     * @param array|string $attrs
     * @return string
     */
    public function render($attrs = []): string
    {
        $user = User::find();
        $customer = ($user ? $user->getCustomer() : null) ?? new Customer();
        $defaultInstrument = $customer->getDefaultPaymentInstrument();
        $defaultCardId = $defaultInstrument instanceof PaymentCardInstrument ? $defaultInstrument->getPaymentCardId() : '';
        $cards = $user ? $user->getCards() : [];
        $actions = [
            'add' => Action::get('rebilly_card_add'),
            'update' => Action::get('rebilly_card_update'),
            'delete' => Action::get('rebilly_card_deactivate'),
        ];

        return renderIsolate(
            REBILLY_WP_DIR . 'template/front/portal.php',
            compact('user', 'customer', 'defaultCardId', 'cards', 'actions'),
            true
        );
    }
}
