<?php
/**
 * @var array $invoices
 */
?>
<?php if (count($invoices) > 0): ?>
    <?php foreach ($invoices as $invoice): ?>
        <div class="rebilly_invoice_wrap">
            <div class="rebilly_invoice status_<?php esc_attr_e($invoice['status']); ?>">
                <div class="name">
                    <?php esc_html_e(sprintf(__('Invoice #%s', REBILLY_WP_LANG_DOMAIN), $invoice['name'])); ?>
                </div>
                <div class="total">
                    <?php esc_html_e(rebilly_money($invoice['amount'], $invoice['currency'])); ?>
                </div>
                <div class="date">
                    <?php if ($invoice['paidTime'] !== false): ?>
                        <?php esc_html_e('Paid at:', REBILLY_WP_LANG_DOMAIN); ?>
                        <span class="text-muted"><?php esc_html_e(date_i18n(get_option('date_format'), $invoice['paidTime'])); ?></span>
                    <?php else: ?>
                        <?php esc_html_e('Due at:', REBILLY_WP_LANG_DOMAIN); ?>
                        <span class="text-muted"><?php esc_html_e(date_i18n(get_option('date_format'), $invoice['dueTime'])); ?></span>
                    <?php endif; ?>
                </div>
                <ul class="item-list">
                    <li class="item header">
                        <div class="title"><?php esc_html_e('Plan', REBILLY_WP_LANG_DOMAIN); ?></div>
                        <div class="unit-price"><?php esc_html_e('Unit price', REBILLY_WP_LANG_DOMAIN); ?></div>
                        <div class="quantity"><?php esc_html_e('Qty.', REBILLY_WP_LANG_DOMAIN); ?></div>
                        <div class="price"><?php esc_html_e('Price', REBILLY_WP_LANG_DOMAIN); ?></div>
                    </li>
                    <?php foreach ($invoice['items'] as $item): ?>
                        <li class="item">
                            <div class="title">
                                <?php esc_html_e($item['name']); ?>
                            </div>
                            <div class="unit-price">
                                <?php esc_html_e(rebilly_money($item['unitPrice'], $invoice['currency'])); ?>
                            </div>
                            <div class="quantity">
                                <?php esc_html_e($item['quantity']); ?>
                            </div>
                            <div class="price">
                                <?php esc_html_e(rebilly_money($item['price'], $invoice['currency'])); ?>
                            </div>
                        </li>
                    <?php endforeach; ?>
                </ul>
                <div class="status <?php esc_attr_e($invoice['status']); ?>">
                    <?php esc_html_e(mb_convert_case(str_replace('-', ' ', $invoice['status']), MB_CASE_TITLE)); ?>
                </div>
                <div class="actions">
                    <a href="<?php echo wp_nonce_url(admin_url("admin-ajax.php?action=rebilly_invoice_download&invoiceId={$invoice['id']}"), 'rebilly_invoice_download', '_nonce'); ?>"
                       class="action download"
                       title="<?php esc_attr_e('Download PDF', REBILLY_WP_LANG_DOMAIN); ?>">
                        <?php echo file_get_contents(REBILLY_WP_DIR . 'images/icon/download.svg'); ?>
                    </a>
                </div>
            </div>
        </div>
    <?php endforeach; ?>
<?php else: ?>
    <div class="rebilly_notice">
        <?php _e('You have not received any invoice yet.', REBILLY_WP_LANG_DOMAIN); ?>
    </div>
<?php endif; ?>
